
/*
===============================================================================
 erpin.c                A.Lambert le 23/12/01, derniere revision le 02/06/11

 - recherche de masques a plusieurs niveaux, dans l'ordre des arg. de 'main'.
 - exploration complete de toutes les sequences.
 - exploration "transparente" des bases d'entrainement.
 - les seuils sont entres en valeurs ou en % de captures dans 'trset'.
 - on dispose de divers styles d'output (LONG | SHORT | MUTE).
 - on traite le brin complementaire, en plus ou separement.
 - on utilise la statistique des sequences analysees (profils de poids stat.).
 - on opere des selections sur les sequences a analyser (position, longueur).
 - on calcule la E-value des detections du dernier motif de la commande,
 - ainsi que l'histogramme des detections a ce meme niveau, option '-hist'.
 - on gere les parametres 'logzero', 'tablen', 'chrono', 'warnings'.
 - les detections repetees et overlap sont elimines avant affichage.
 - les masques (motifs) successifs sont geres de facon statique ou dynamique.
 - un controle des besoins en memoire precede les operations de recherche.
 - des matrices de substitution peuvent etre introduites, lesquelles modifient
   les profils des brins et/ou helices.
 - L'affichage de l'état d'avancement d'un job est débranché par '-silent'.

 make -f apps.mk erpin ;
 mv erpin ../bin ;
 ou
 cc -O3 -Wall -o erpin erpin.c -I../include -L../lib -lrnaIV -lm ;
 chmod 755 erpin ;
 strip erpin ;

 erpin <trset> <data> <region>
       -nomask|((-mask|-umask|-add) <elt1> <elt2>..)      level1,
       [-nomask|((-mask|-umask|-add) <elt1> <elt2>..)]    level2, default: void
       [...]                                              level..          idem
       [-cutoff <cutoff1> <cutoff2> ..]                           default: 100%
       [-dmp|-smp]                                                default: -dmp
       [-fwd|-rev|-fwd+rev]                                   default: -fwd+rev
       [-long|-short|-mute]                                      default: -long
       [-warnings]                                                 default: OFF
       [-globstat|-locstat|-unifstat]                        default: -globstat
       [-Eon|-Eoff]                                               default: -Eon
       [-hist]                                                     default: OFF
       [-seq1 <seqnb1>][-nseq <nseq>]               defaut: 1, SEQ_MAX_NB (all)
       [-bgn <seqbgn>][-len <range>]                     defaut: 1, SEQ_MAX_LEN
       [-logzero <logzero>]                                        default: -20
       [-tablen <tablen>]                                         default: 1024
       [-chrono]                                                   default: OFF
       [-silent]           débranche l'état d'avancement d'un job,  default: ON

       [-pcw <pcw>]                       poids des pseudo-comptes, defaut: 0.1
       [-hpcw <hpcw>]    poids des pseudo-comptes pour les helices, defaut: 0.1
       [-spcw <spcw>]      poids des pseudo-comptes pour les brins, defaut: 0.1
       [-sumf <fname>]        fichier contenant des mat. de subst. alternatives

 default command line: "erpin5 <trset> <data> <region> <mask>" stands for:

 erpin <trset> <data> <region> <mask> -cutoff 100% -fwd+rev -long -globstat \
       -seq1 1 -nseq 10M -bgn 1 -len 300M \
       -pcw 0.1 -logzero -20. -tablen 1024 -dmp ;

 erpin ~/devc/projets/bio/data/trsetsII/trna-typeI.epn \
       ~/devc/projets/bio/data/sequences/ecoli.fna -2,2 \
       -cutoff 100% 100% 90% \
       -umask 6 8 -mask 2 3 -nomask -chrono -short -logzero -30 ;

 erpin ~/devc/projets/bio/data/trsetsII/trna-typeI.epn \
       ~/devc/projets/bio/data/sequences/borrelia.fasta -2,2 \
       -cutoff 100% 100% 90% \
       -umask 6 8 -mask 2 3 -nomask -chrono ;

 erpin ~/devc/projets/bio/data/trsetsII/trna-typeI.epn \
       ~/devc/projets/bio/data/sequences/Chr_22_01-12-1999.fasta -2,2 \
       -cutoff 100% 100% 90% \
       -umask 6 8 -mask 2 3 -nomask -chrono -short -logzero -30 ;

 erpin ~/devc/projets/bio/data/trsetsII/trna-typeI.epn \
       ~/devc/projets/bio/data/sequences/ecoli.fna -2,2 \
       -cutoff 100% 100% 90% \
       -umask 6 8 -mask 2 3 -nomask -fwd -chrono -short -logzero -30 \
       -sumf ../sum/SUM.dat ;

 erpin ~/devc/projets/bio/data/trsetsII/trna.db \
       ~/devc/projets/bio/data/sequences/ecoli.fna -2,2 \
       -cutoff 100% 100% 100% \
       -umask 7 20 -mask 2 3 -nomask -fwd -chrono -smp -short -logzero -30 ;

 erpin ~/devc/projets/bio/data/trsetsIII/LET-7.epn \
       ~/devc/projets/bio/data/sequences/randseq10M.fna -2,2 \
       -umask 2 4 6 8 3 -cutoff -39 -sumf ../sum/SUM.dat -fwd \
       -chrono -mute -logzero -30 ;

E-value at cutoff -39.0 for 10.0Mb single strand data: 46.2e+01
37 hits

 erpin ~/devc/projets/bio/data/trsetsIII/LET-7.epn \
       ~/devc/projets/bio/data/sequences/randseq10M.fna -6,6 \
       -umask 6 -sumf ../sum/SUM.dat -fwd -chrono -mute -logzero -30 ;

E-value at cutoff 13.1 for 10.0Mb single strand data: 2.93e+01
21 hits

 erpin ~/devc/projets/bio/data/trsetsIII/LET-7.epn \
       ~/devc/projets/bio/data/sequences/randseq10M.fna -6,6 -umask 6 \
       -cutoff 5 -fwd -chrono -mute -logzero -30 ;

E-value at cutoff 5.0 for 10.0Mb single strand data: 2.28e+02
230 hits

erpin ~/devc/projets/bio/data/trsetsII/trna.db \
      ~/devc/projets/bio/data/sequences/toto.fna  8,8 -umask 8 -chrono ;

erpin ~/devc/projets/bio/data/trsetsIII/rnasep2.epn \
      ~/devc/projets/bio/data/essai431.fna 67,31 \
      -umask 9 61 -add 65 -add 15 \
      -cutoff 90% 90% 90% -fwd -chrono -short ;

erpin ~/devc/projets/bio/data/trsetsIII/rnasep2.epn \
      ~/devc/projets/bio/data/bank 67,69 -umask 67 -add 58 -add 63 \
      -cutoff 90% -fwd -chrono -short ;

erpin ~/devc/projets/bio/data/trsetsIII/rnasep2.epn \
      ~/devc/projets/bio/data/essai431.fna 67,69 -umask 67 -add 58 -add 63 \
      -cutoff 90% 90% 60 -fwd -chrono -short ;

erpin ~/devc/projets/bio/data/trsetsIII/rnasep2.epn \
      ~/devc/projets/bio/data/essai431.fna 67,69 \
      -umask 67 -add 58 -add 63 -add 36 \
      -cutoff 90% 90% 90% 70 -fwd -chrono -short -Eoff ;

erpin ~/devc/projets/bio/data/trsetsIII/rnasep2.epn \
      ~/devc/projets/bio/data/essai431.fna 67,69 \
      -umask 67 -add 58 -add 63 -add 36 -add 57 \
      -cutoff 90% 90% 90% 90% 55 -fwd -chrono -short -Eoff ;

erpin ~/devc/projets/bio/data/trsetsII/gcvt.epn \
      ~/devc/projets/bio/data/sequences/rnd100K.epn \
       -2,2 -umask 3 -add 2 -add 4 -add 6 8 10 -fwd ;

erpin ~/devc/projets/bio/data/trsetsII/gcvt.epn \
      ~/devc/projets/bio/data/sequences/rnd100K.epn \
       -2,2 -umask 3 -add 2 -add 4 -add 6 -fwd ;

erpin ~/devc/projets/bio/data/trsetsII/gcvt.epn \
      ~/devc/projets/bio/data/sequences/randseq10M.fna \
       -2,2 -umask 3 -add 2 -add 4 -add 6 -fwd -short -chrono ;

erpin ~/devc/projets/bio/data/trsetsII/RF00006.epn \
      ~/devc/projets/bio/data/trsetsII/RF00006.epn 1,2 \
      -umask 2 4 -add 6 -add 8 -fwd -Eoff ;

erpin ~/devc/projets/bio/data/trsetsII/RF00006.epn \
      ~/devc/projets/bio/data/trsetsII/RF00006.epn 1,2 \
      -umask 2 4 -add 6 -add 8 -cutoff 16 25 32 -fwd -Eoff ;

erpin ~/devc/projets/bio/data/trsetsII/RF00006.epn \
      ~/devc/projets/bio/data/sequences/randseq1M.fna 1,2 \
      -umask 2 4 -add 6 -add 8 -cutoff 16 25 32 -fwd -Eoff ;

erpin ~/devc/projets/bio/data/trsetsII/RF00008.N.epn \
      ~/devc/projets/bio/data/trsetsII/RF00008.N.epn 1,17 \
      -umask 11 8 -add 2 -add 15 -add 4 -add 9 -fwd \
      -cutoff 102% 101% 101% 101% 101% ;

===============================================================================
 test de '-silent' qui débranche l'affichage de l'état d'avancement d'un job

 erpin ~/devc/projets/bio/data/trsetsII/trna-typeI.epn \
       ~/devc/projets/bio/data/sequences/randseq10M.fna -2,2 \
       -umask 6 8 -mask 2 3 -nomask -chrono -short ;

 erpin ~/devc/projets/bio/data/trsetsII/trna-typeI.epn \
       ~/devc/projets/bio/data/sequences/randseq10M.fna -2,2 \
       -umask 6 8 -mask 2 3 -nomask -chrono -short -silent ;

===============================================================================
*/

#include "rnaIV.h"
/*
#include <mcheck.h>
*/
void ErpinHelp(void);

/*
===============================================================================
*/

int main(int argc, char *argv[])
{
  FILE      *txt;
  Sequence  *seq;
  Threshold *thresholds;
  Trset     *trset;
  Pattern   *pattern;
  Mask      *mask;
  Context   *ctxt;
  int       nmask, style, dir, stat, warnings, chrono, maskproc,
            first, nseq, bgn, range, eval, hist,
            total = 0;
  double    datavol;

  int       pcmode;               /* variables concernant les pseudo-comptes */
  double    hpcw, spcw;
  char      *sumfname;

  /*--------------- lecture des arguments -----------------------------------*/

  ReadHelpArgs(argc, argv, 4, ErpinHelp);
  fTest(argv[1]);
  fTest(argv[2]);

  mask = ReadMasksArgs(argc, argv, &nmask);

  thresholds = GetThresholdsArgs(argc, argv, nmask);
  GetEnvArgs(argc, argv, &chrono);
  GetProcArg(argc, argv, &maskproc);
  GetOutputArgs(argc, argv, &style, &warnings, &eval, &hist);
  GetDirArg(argc, argv, &dir);
  GetStatArg(argc, argv, &stat);
  GetSeqArgs(argc, argv, &first, &nseq, &bgn, &range);
  sumfname = GetSumArgs(argc, argv, &pcmode, &hpcw, &spcw);

  /*----------------- Trset, Pattern & Masques, Profils  --------------------*/

  trset = ReadTrset(argv[1], 'S', stdout);                          /* Trset */
  pattern = ReadPattern(argv[3], trset);                          /* Pattern */
  ParseMasksArgs(mask, nmask, pattern);
  InitStatTables();                             /* distr. uniforme du "fond" */

  SetNtCodes();        /* tableau de codage des nt pour le calcul des scores */

  if (pcmode != PC_OFF) {                                  /* pseudo-comptes */
      ReadSUM(sumfname, pcmode, hpcw, spcw, trset);
      GetPatternProfilesSM(trset, pattern);                      /* Profiles */
  }
  else GetPatternProfiles(trset, pattern);                       /* Profiles */

  GetMasks(mask, nmask, pattern);                                   /* Masks */

                                                    /* Ctrl du nb de configs */
  CtrlCfgs(mask, nmask, maskproc, 'S', stdout);        /* avant le demarrage */
                                                   /* du calcul des tableaux */

  if (maskproc == DYNAMIC) GetMaskCfgs(mask);       /* Configs du 1er masque */
  else GetMasksCfgs(mask, nmask);             /* Configs de tous les masques */

  /*--------------- Seuils supposant une database uniforme ------------------*/

  GetThresholds(thresholds, trset, mask, nmask);               /* Thresholds */
  fPrintfThresholds(mask, nmask, stdout);            /* affichage des seuils */
  DelTrset(trset);

  /*------------------- Statistique sur la Database -------------------------*/

  fprintf(stdout, "Database:\t\"%s\"\n", argv[2]);

  if (stat == GLOBAL) {
      datavol = fGetLongStat(argv[2], first, nseq, bgn, range,
                             mask[nmask-1].min_len, 'V', stdout);
      ChPatternStat(pattern);
  }
  else
      datavol = fGetShortStat(argv[2], first, nseq, bgn, range,
                              mask[nmask-1].min_len, 'V', stdout);

  /*------------- Calcul de la E-value du masque principal ------------------*/

  if (eval == ON) {
      double E, cutoff = mask[nmask-1].threshold;
      GetEvalues(mask + nmask - 1, datavol, dir, stat);
      E = Evalue(cutoff);
      fPrintfEvalue(stdout, E, cutoff, dir, datavol);
  }

  /*--------------- Sequences et Contextes ----------------------------------*/

  seq = InitSeq(GetTail(pattern));                              /*  Sequence */
  ctxt = SetupContext(mask, nmask);                             /* Contextes */
  InitSeqContexts(ctxt, nmask, seq);
  StoreInterval(ctxt, bgn, range);
  InitOutputContexts(ctxt, nmask, style, hist, eval, warnings, chrono, stdout);
  SetMasksScoresTabs(ctxt, nmask);                             /* Score Tabs */

  /*--------------- Exploration des Sequences -------------------------------*/

  StartStatus();           /* debut de l'affichage du nbre de sites explores */
  StartTimer(ctxt);                                   /* et du chronometrage */

  txt = fopen(argv[2], "r");

  while ( ReadSeq(txt, seq) != 0 )                                 /* Search */
  {
      if (seq->nb >= first && seq->nb < first + nseq)
      {
          PreProcessSeq(seq);

	  if (stat == UNIFORM) {
              if (dir == FORWARD || dir == FWD_AND_REV) {
                  ResetDetect(ctxt, nmask);
                  SetDir(ctxt, nmask, FORWARD);
                  total += Search(ctxt, nmask, maskproc);
              }
              if (dir == REV_CMPL || dir == FWD_AND_REV) {
                  ResetDetect(ctxt, nmask);
                  SetDir(ctxt, nmask, REV_CMPL);
                  RevCmpl(seq);
                  total += Search(ctxt, nmask, maskproc);
              }
	  }
          else                      /* stat est alors egal a LOCAL ou GLOBAL */
	  {
              if (stat == LOCAL) {
                  sGetStat(seq->data, seq->datalen);
                  ChPatternStat(pattern);
              }
              if (dir == FORWARD || dir == FWD_AND_REV) {
                  ResetDetect(ctxt, nmask);
                  SetDir(ctxt, nmask, FORWARD);
#ifdef DEBUG
Pause();
#endif
		  total += Search(ctxt, nmask, maskproc);
              }
              if (dir == REV_CMPL || dir == FWD_AND_REV) {
                  ResetDetect(ctxt, nmask);
                  SetDir(ctxt, nmask, REV_CMPL);
                  RevCmpl(seq);
                  ReverseStat();
                  ChPatternStat(pattern);
                  total += Search(ctxt, nmask, maskproc);

                  if (stat == GLOBAL) {        /* restaure les freq. du fond */
                      ReverseStat();
                      ChPatternStat(pattern);
                  }
	      }
          }
      }
  }

  fclose(txt);
  fPrintfProcessLog(ctxt, nmask, total);

  /*-------------------- Sortie ---------------------------------------------*/

  if (eval == ON) FreeEvalsTab();
  DelMasks(mask, nmask);
  DelPattern(pattern);
  DelSeq(seq);
  free(ctxt);
  free(thresholds);
  FreeNtCodes();

  exit(0);
}
/*
===============================================================================
 ErpinHelp(): help
===============================================================================
*/

void ErpinHelp(void)
{
  fprintf(stderr,
"\n"
"========== erpin: Easy Rna Profile IdentificatioN, Version 5.5.4 ===========\n\n"

"Usage:\n"
"erpin [-h]                                                              help\n"
"erpin <training-set>                                  training set file name\n"
"      <input-file>                                database file name (fasta)\n"
"      <region>                                            region of interest\n"
"      -nomask|((-mask|-umask|-add) <elt1> ...)      level1,\n"
"      [-nomask|((-mask|-umask|-add) <elt1> ...)]    level2,    default: void\n"
"      [...]                                         level..             idem\n"
"      [-cutoff <cutoff1> <cutoff2> ..]                         default: 100%%\n"
"      [-dmp|-smp]                                              default: -dmp\n"
"      [-fwd|-rev|-fwd+rev]                                 default: -fwd+rev\n"
"      [-long|-short|-mute]                                    default: -long\n"
"      [-warnings]                                               default: OFF\n"
"      [-globstat|-locstat|-unifstat]                      default: -globstat\n"
"      [-Eon|-Eoff]                             E-value ON|OFF, default: -Eon\n"
"      [-hist]                                                   default: OFF\n"
"      [-seq1 <seqnb1>][-nseq <nseq>]            default: 1, SEQ_MAX_NB (all)\n"
"      [-bgn <seqbgn>][-len <range>]                  default: 1, SEQ_MAX_LEN\n"
"      [-logzero <logzero>]                                      default: -20\n"
"      [-tablen <tablen>]                                       default: 1024\n"
"      [-chrono]                                                 default: OFF\n"
"      [-silent]               switch off job progression status, default: ON\n"
"      [-pcw <pcw>]                        pseudo-counts weight, default: 0.1\n"
"      [-hpcw <hpcw>]          pseudo-counts weight for helices, default: 0.1\n"
"      [-spcw <spcw>]          pseudo-counts weight for strands, default: 0.1\n"
"      [-sumf <fname>]   file of alternate user defined substitution matrices\n\n"

"default command line: \"erpin <trset> <data> <region> <mask>\" stands for:\n\n"

"erpin <trset> <data> <region> <mask> -cutoff 100%% \\ \n"
"      -dmp -fwd+rev -long -globstat -Eon \\ \n"
"      -seq1 1 -nseq 10M -bgn 1 -len 300M -pcw 0.1 -logzero -20 -tablen 1024\n\n"

"Example:\n"
"erpin trsets/trna.epn  sequences/ecoli.fna -2,2 -umask 20 11 -nomask \\ \n"
"      -cutoff 100%% 90%% -sumf ../sum/SUM.dat -fwd -short -chrono\n\n"

"============= D.Gautheret A.Lambert, Luminy Marseille, May 2006 ============\n\n"
);

  return;
}
/*
===============================================================================
*/
