
/*
===============================================================================
 mcfgs.c                           A.Lambert le 25/05/05  revu le 03/03/06

 code des fonctions destinees au calcul du nombre de configurations des gaps
 d'un masque et a en dresser la liste.
 On prend en compte les groupes de brins exterieurs au masque mais appartenant
 a son enveloppe, et les groupes de brins appartenant au masque et dont la som-
 me des nombres de gaps est connu.

 cc -O2 -Wall -c mcfgs.c -DDEBUG -I../include ;            aide au deboggage

 cc -O2 -Wall -c mcfgs.c -I../include ;
 ar -rs ../lib/librnaIV.a mcfgs.o ;

===============================================================================
*/

#include "rnaIV.h"

int   StressCfgsNb(short **gaplist, int width, int *col, int *elts);
int   GroupCfgsNb(short **gaplist, int width, int *col);
void  DmpCfgsNb(Mask *mask);
short **GetStressCfgs(short **gaplist, int width, int *col, int *cfgs, int *cfgw);
int   GetGapsCfgs(Mask *mask);
void  EnumCfgs(GapsGrp *grp, int ngr, int col, int *nb, int **dims, short *tmpcfg,
               short ***StressCfgs, short **cfglist);

void  PrtGapsCfg(Mask *mask, int cfgindex, FILE *txt);
void  PrtGapsCfgs(Mask *mask, FILE *txt);

void  GetCfg(Config *cfg, Mask *mask, short *gaps_cfg);
void  GetMaskCfgs(Mask *mask);

int   GetMaskCfgLen(Config *cfg, Mask *mask);
void  GetMaskCfgsLens(Mask *mask);

/* fonctions de 'libsrc/conv.c' et 'libsrc/tab1.c' */
int    *iconvlv(int *a, int *b, int na, int nb, int *ncnv);
void   iconv(int **a, int *na, int *b, int nb);
int    *ipadd(int bgn, int len);

/*
===============================================================================
 StressCfgsNb(): La fonction utilise le tableau pointe par 'gaplist' de largeur
                 'width'. Elle retourne le nombre de configurations de gaps du
                 groupe de brins associe a la colonne pointee par 'col' en en-
		 tree. En sortie 'col' pointe la colonne qui suit immediatement
		 le groupe qui a ete traite, et 'elts' pointe le nombre d'ele-
		 ments (groupes) affectes par la contrainte consideree.
		 Le resultat est obtenu par iteration du produit de convolution
		 de fonctions caracteristiques d'intervalles.
		 Le tableau pointe par 'gaplist' est suppose representer le
		 champ 'gapslist' correctement initialisee d'une structure de
		 type 'Mask', cette fonction sera interfacee dans ce contexte.
===============================================================================
*/
int StressCfgsNb(short **gaplist, int width, int *col, int *elts)
{
  int I, j, bgn, *a, *b, na, nb, stress_indx, gr_indx, row, w, sum, cfgs;

  stress_indx = gaplist[STR_INDEX][*col];
  I = MGAPLIST_H + stress_indx;                    /* ligne de la contrainte */

  if (gaplist[I][*col] != 1)
      return 0;                                               /* pas d'objet */

  sum = gaplist[I][width + 1];       /* lecture de la somme des gaps imposee */

  w = gaplist[STATUS][*col];
  row = (w == 1 ? AT_MINGAPS : GR_MINGAPS);
  bgn = gaplist[row][*col];
  na = gaplist[row + 1][*col] + 1;
  a = ipadd(bgn, na);                         /* fonction caracteristique de */
                                                   /* l'intervalle [bgn, na] */
  *col += w;                           /* colonne associee au groupe suivant */
  gr_indx = gaplist[GR_INDEX][*col];

  for (*elts = 1, j = *col; j < width && gr_indx != -1; j += w)
  {
      gr_indx = gaplist[GR_INDEX][j];
      w = gaplist[STATUS][j];
      if (gaplist[I][j] == 1)
      {
          (*elts)++;        /* nombre d'elements impliques par la contrainte */
          *col += w;                   /* colonne associee au groupe suivant */
          row = (w == 1 ? AT_MINGAPS : GR_MINGAPS);
          bgn = gaplist[row][j];
          nb = gaplist[row + 1][j] + 1;
          b = ipadd(bgn, nb);         /* fonct. caracteristique de [bgn, nb] */

          iconv(&a, &na, b, nb);
          free(b);
      }
  }
  cfgs = a[sum];
  free(a);

#ifdef DEBUG
fprintf(stderr, "StressCfgsNb: na=%d, sum=%d\n", na, sum);
#endif

  return cfgs;
}
/*
===============================================================================
 GroupCfgsNb(): variante de la fonction 'StressCfgsNb' qui etend son action aux
                divers cas qui peuvent se presenter:
		1/ index du groupe == -1, colonne exterieure a l'enveloppe du
		masque associe a 'gaplist'. 'col' avance jusqu'a un index != -1
		2/ index du groupe != -1
		    A/ stress_index == -1,
		        a/ status == 1, le groupe est une colonne isolee
			b/ status > 1, le groupe forme un ensemble exterieur
			   au masque, les gaps sont concatenes.
			   Dans les 2 cas '*col' avance de la largeur de groupe.
		    B/ stress_index != -1, marque le debut d'une contrainte
		       voir 'StressCfgsNb'.
===============================================================================
*/
int GroupCfgsNb(short **gaplist, int width, int *col)
{
  int I, j, bgn, *a, *b, na, nb, stress_indx, gr_indx, row, w, sum, cfgs;

  gr_indx = gaplist[GR_INDEX][*col];
  if (gr_indx == -1)                 /* colonne ext. a l'enveloppe du groupe */
  {
      while (gaplist[GR_INDEX][*col] == -1) (*col)++;
      return 1;                                            /* --- sortie --- */
  }

  stress_indx = gaplist[STR_INDEX][*col];
  if (stress_indx == -1)            /* colonne isolee ou groupe a concatener */
  {
      w = gaplist[STATUS][*col];
      row = (w == 1 ? AT_MINGAPS : GR_MINGAPS);
      cfgs = gaplist[row + 1][*col] - gaplist[row][*col] + 1;
      *col += w;
      return cfgs;                                         /* --- sortie --- */
  }
                      /* sinon, colonne debutant l'expression d'1 contrainte */

  I = MGAPLIST_H + stress_indx;                    /* ligne de la contrainte */
  sum = gaplist[I][width + 1];       /* lecture de la somme des gaps imposee */

  w = gaplist[STATUS][*col];
  row = (w == 1 ? AT_MINGAPS : GR_MINGAPS);
  bgn = gaplist[row][*col];
  na = gaplist[row + 1][*col] + 1;
  a = ipadd(bgn, na);                         /* fonction caracteristique de */
                                                   /* l'intervalle [bgn, na] */
  *col += w;                           /* colonne associee au groupe suivant */
  gr_indx = gaplist[GR_INDEX][*col]; /* par construction: au moins 2 groupes */

  for (j = *col; j < width && gr_indx != -1; j += w)
  {
      gr_indx = gaplist[GR_INDEX][j];
      w = gaplist[STATUS][j];
      if (gaplist[I][j] == 1)
      {
          *col += w;                   /* colonne associee au groupe suivant */
          row = (w == 1 ? AT_MINGAPS : GR_MINGAPS);
          bgn = gaplist[row][j];
          nb = gaplist[row + 1][j] + 1;
          b = ipadd(bgn, nb);         /* fonct. caracteristique de [bgn, nb] */

          iconv(&a, &na, b, nb);
          free(b);
      }
  }
  cfgs = a[sum];
  free(a);

  return cfgs;
}
/*
===============================================================================
 DmpCfgsNb(): Calcule le nombre de configurations de gaps du masque pointe par
              'mask' en iterant la fonction 'GroupCfgsNb'.
	      Le resultat est retourne dans les champs 'mask->log2ncfg' (en log
	      base 2) et 'mask->ncfg'.
	      Le champ 'gapslist' de l'argument 'mask' est suppose completement
	      initialise.
	      Cette fonction ne sera utilisee que dans '/libsrc/ctrlcfgs.c' pour
	      une estimation a priori du nombre de configs necessaires (et des
	      besoins en espace memoire).
===============================================================================
*/
void DmpCfgsNb(Mask *mask)
{
  int    j, width, n;
  long long nb;
  double log2ncfg;

  width = mask->pattern->nvarst;
  if (width == 0) {
      mask->log2ncfg = 0.;
      mask->ncfg = 1;                                     /* 0 gap: 1 config */
      return;
  }

  log2ncfg = 0.0;
  nb = 1;
  for (j = 0; j < width;  )
  {
      n = GroupCfgsNb(mask->gapslist, width, &j);
      log2ncfg += LOG2(n);
      nb *= (long long) n;
  }
  mask->log2ncfg = log2ncfg;
  mask->ncfg = 0;

  if (log2ncfg <= CFG_MAX_BITS)  mask->ncfg = (int) nb;

  return;
}
/*
===============================================================================
 GetStressCfgs(): Retourne un pointeur sur la liste des configurations denombre-
                  es par 'StressCfgsNb', le nombre de configurations est pointe
		  en retour par 'cfgs', et 'cfgw' pointe la largeur du tableau.
		  'col' pointe en entreee la colonne traitee et en sortie celle
		  qui suit immediatement le groupe qui a ete traite.
		  Si 'cfgs' est nul la fonction retourne un  pointeur NULL.
		  un arg. 'col' errone provoque la sortie commentee du prog.
===============================================================================
*/
short **GetStressCfgs(short **gaplist, int width, int *col, int *cfgs, int *cfgw)
{
  int   I, j, n, stress_indx, gr_indx, tmp_col, row, w, sum;
  short **cfglist, *A, **T, s;

  gr_indx = gaplist[GR_INDEX][*col];
  stress_indx = gaplist[STR_INDEX][*col];
  I = MGAPLIST_H + stress_indx;                    /* ligne de la contrainte */

  if (gr_indx == -1 || gaplist[I][*col] != 1) {               /* pas d'objet */
      fprintf(stderr, "GetStressCfgs: invalid arg #3, exit..\n");
      exit(1);
  }
  tmp_col = *col;
  *cfgs = StressCfgsNb(gaplist, width, &tmp_col, cfgw);

#ifdef DEBUG
fprintf(stderr, "GetStressCfgs: cfgs=%d\n", *cfgs);
#endif

  if (*cfgs == 0) return NULL;                             /* --- sortie --- */

  cfglist = sMat(*cfgs, *cfgw);                /* tableau des configurations */
  A = (short *) malloc(*cfgw * sizeof(short));  /* tableau d'1 configuration */
  T = sMat(2, *cfgw);                      /* tableau optimisant l'adressage */
                                                /* des elements de 'gaplist' */
  sum = gaplist[I][width + 1];       /* lecture de la somme des gaps imposee */

  for (j = 0; j < *cfgw; j++)                           /* tableau simplifie */
  {
      w = gaplist[STATUS][*col];
      row = (w == 1 ? AT_MINGAPS : GR_MINGAPS);
      T[0][j] = gaplist[row][*col];
      T[1][j] = gaplist[row + 1][*col];
      *col += w;
  }

#ifdef DEBUG
printf("ctrl GetStressCfgs: %d %d, si = OK\n", *col, tmp_col);
#endif
                         /* le tableau T est actualise en vue d'eliminer les */
                                      /* configurations impossibles a priori */
  {
    int tmp, min, max;

    for (min = max = j = 0; j < *cfgw; j++) {
        min += T[0][j];
        max += T[1][j];
    }
			                    /* les combinaisons trop grandes */
    for (j = 0; j < *cfgw; j++) {
        tmp = min - T[0][j];
        if (tmp + T[1][j] > sum)
            T[1][j] = MAX(T[0][j], sum - tmp);
    }
                                            /* les combinaisons trop petites */
    for (j = 0; j < *cfgw; j++) {
        tmp = max - T[1][j];
        if (tmp + T[0][j] < sum) {
            T[0][j] = MIN(T[1][j], sum - tmp);
        }
    }
  }

  for (j = 0; j < *cfgw; j++)  A[j] = T[0][j];         /* 1ere configuration */

  for (n = 0; n < *cfgs; )       /* enumeration de TOUTES les configurations */
  {                                             /* associees a la contrainte */
      for (s = j = 0; j < *cfgw; j++) s += A[j];
      if (s == sum) {                                           /* selection */
          for (j = 0; j < *cfgw; j++) cfglist[n][j] = A[j];    /* enregistr. */
	  n++;
      }                                                  /* config. suivante */
      for (j = 0; j < *cfgw && ++A[j] > T[1][j]; j++) {
          A[j] = T[0][j];
#ifdef DEBUG                                      /* un souvenir de 02/06 .. */
if (j >= *cfgw) fprintf(stderr, "## Warning: (j >= *cfgw): %d %d !\n", j, *cfgw);
#endif
      }
  }
  free(A);
  FreesMat(T);

  return cfglist;
}
/*
===============================================================================
 GetGapsCfgs(): Dresse la liste des configurations denombrees pour le masque
                pointe par 'mask' dont le champ 'gapslist' est suppose comple-
		tement initialise, c'est un tableau de hauteur 'mask->ncfg'
		(valeur aussi retournee par la fonction, pour memoire) et de
		largeur 'mask->ngrps'.
	        Interface de 'EnumCfgs'.
		On notera que 'mask->ncfg' et la valeur retournee sont calculees
		de 2 facons independantes (calcul a l'aide des produits de con-
		volution et enumeration directe), permettant ainsi un controle
	        du resultat obtenu.
	        Celui-ci est explicitement demande dans la fonction 'GetMaskCfgs'
		dont la construction est le but poursuivi par 'GetGapsCfgs'.
===============================================================================
*/
int GetGapsCfgs(Mask *mask)
{
  int    j, stress_indx, gr_indx, w, width, row, total, cfgs, **dims;
  short  ***StressCfgs, *tmpcfg;

  dims = NULL;
  StressCfgs = NULL;
  tmpcfg = NULL;

  if (mask->ngrps == 0) {           /* pas de brins avec gaps pour ce masque */
      total = mask->ncfg = 1;                     /* une seule configuration */
      SetGapCfgTable(mask);
      mask->gapscfg[0][0] = 0;
      return total;                 /* --- sortie retournant la valeur 1 --- */
  }
       /* sinon, il y a des brins avec gaps dans ce masque: mask->ngrps != 0 */

  width = mask->pattern->nvarst;

  if (mask->nstress > 0)
  {
      dims = iMat(2, mask->nstress);
      StressCfgs = (short ***) malloc(mask->nstress * sizeof(short **));
  }
  tmpcfg = (short *) calloc(mask->ngrps, sizeof(short));

  for (total = 1, j = 0; j < width; )
  {
      gr_indx = mask->gapslist[GR_INDEX][j];

      if (gr_indx == -1) {           /* colonne ext. a l'enveloppe du groupe */
          while (mask->gapslist[GR_INDEX][j] == -1) j++;
          continue;
      }

      stress_indx = mask->gapslist[STR_INDEX][j];

      if (stress_indx == -1)        /* colonne isolee ou groupe a concatener */
      {
          w = mask->gapslist[STATUS][j];
          row = (w == 1 ? AT_MINGAPS : GR_MINGAPS);
          cfgs = mask->gapslist[row + 1][j] - mask->gapslist[row][j] + 1;
          j += w;
          total *= cfgs;
	  continue;
      }

#ifdef DEBUG
{
static int call=0;
fprintf(stderr, "GetGapsCfgs I: call=%d, stress_indx=%d, j=%d, w=%d\n",
                 call++, stress_indx, j, w);
}
#endif
                                   /* note: en entree 'w' n'est pas utilisee */
      StressCfgs[stress_indx] =
                      GetStressCfgs(mask->gapslist, width, &j, &cfgs, &w);

#ifdef DEBUG
fprintf(stderr, "GetGapsCfgs II: cfgs=%d, w=%d\n", cfgs, w);
#endif

      dims[0][stress_indx] = cfgs;             /* dimensions du tableau cree */
      dims[1][stress_indx] = w;         /* GetStressSize(mask, stress_indx); */
      total *= cfgs;
  }

#ifdef DEBUG
printf("GetCfgs: total=%d\n", total);
printf("GetCfgs: w=%d\n", w);
#endif

  mask->ncfg = total;              /* total >= 1: au moins une configuration */
  SetGapCfgTable(mask);         /* allocation du tableau des configs de gaps */

    /*
       ##### la fonct. recursive dressant le tableau final des cfgs. #####
       voir 'SetGapCfgTable' dans libsrc/cfg.c
       'mask->gapscfg' y est cree, il faudra dorenavant le creer a ce niveau
        car 'mask->ncfg' n'est plus connu a l'avance, de largeur 'mask->ngrps'
    */

  total = 0;

  EnumCfgs(mask->grp, mask->ngrps, 0, &total, dims, tmpcfg, StressCfgs, mask->gapscfg);

  if (mask->nstress > 0)
  {
      FreeiMat(dims);
      for (j = 0; j < mask->nstress; j++) FreesMat(StressCfgs[j]);
      free(StressCfgs);
  }
  free(tmpcfg);

#ifdef DEBUG
printf("\n#configs: %d expected, %d found\n", mask->ncfg, total);
#endif

  return total;
}
/*
===============================================================================
 EnumCfgs(): fonction recursive de calcul des configurations de gaps.
             L'argument 'col' indique le numero de colonne dans le tableau des
	     groupes (et non pas des brins !).
	     La fonction suppose que les tableaux pointes par 'gaplist' et
	     'grp' ne sont pas vides (voir 'GetGapsCfgs()').
===============================================================================
*/
void EnumCfgs(GapsGrp *grp, int ngr, int col, int *nb, int **dims, short *tmpcfg,
              short ***StressCfgs, short **cfglist)
{
  short  i, j, size, stress_num, min, max;

  if ((stress_num = grp[col].stress_num) != -1)    /* debut d'une contrainte */
  {
      min = 0;
      max = dims[0][stress_num] - 1;          /* contrainte: nbre de configs */
      size = dims[1][stress_num];      /* nb. de groupes impliques, size > 1 */

#ifdef DEBUG
printf("### EnumCfgs: ngr=%d\n", ngr);
printf("### EnumCfgs: col=%d\n", col);
printf("### EnumCfgs: stress_num=%d\n", stress_num);
printf("### EnumCfgs: max=%d\n", max);
printf("### EnumCfgs: size=%d\n\n", size);
#endif

  }
  else {
      min = grp[col].min_gaps;
      max = grp[col].max_gaps;
      size = 1;
  }
  for (i = min; i <= max; i++)
  {
      if (size == 1)  tmpcfg[col] = i;
      else
      for (j = 0; j < size; j++)  tmpcfg[col+j] = StressCfgs[stress_num][i][j];

      if (col + size == ngr)                    /* derniere colonne atteinte */
      {                                /* enregistrement de la configuration */
          for (j = 0; j < ngr; j++)  cfglist[*nb][j] = tmpcfg[j];
          (*nb)++;
      }
      else
      EnumCfgs(grp, ngr, col + size, nb, dims, tmpcfg, StressCfgs, cfglist);
  }
  return;
}
/*
===============================================================================
 PrtGapsCfg(): Imprime le contenu de la configuration de gaps No 'cfgindex'
               du masque pointe par 'mask' dans le fichier pointe par 'txt'.
===============================================================================
*/
void PrtGapsCfg(Mask *mask, int cfgindex, FILE *txt)
{
  int j;

  for (j = 0; j < mask->ngrps; j++)
      fprintf(txt, "%-2d ", mask->gapscfg[cfgindex][j]);
  fprintf(txt, "\n");

  return;
}
/*
===============================================================================
 PrtGapsCfgs(): Imprime le contenu du tableau des configurations de gaps du
                masque pointe par 'mask' dans le fichier pointe par 'txt'.
===============================================================================
*/
void PrtGapsCfgs(Mask *mask, FILE *txt)
{
  int i;

  for (i = 0; i < mask->ncfg; i++)  PrtGapsCfg(mask, i, txt);
  fprintf(txt, "\n");

  return;
}
/*
===============================================================================
 On passe des configurations de gaps aux configurations completes du masque..
===============================================================================

===============================================================================
 GetCfg(): Retourne, pointee par 'cfg', la configuration du masque pointe par
           'mask' calculee a l'aide des parametres lus dans la configuration de
           gaps pointee par 'gaps_cfg'.
           Le debut de la configuration a pour abscisse zero.

           La configuration, ainsi que 'gaps_cfg', prennent en compte de facon
           appropriee tous les atomes de l'enveloppe du masque, meme ceux qui
           ne lui appartiennent pas.

   rappel: atom[i].h2index vaut 0 ou, dans le cas ou atom[i].type = HLX1 (1er
           brin d'helice), a pour valeur l'indice de l'atome du 2eme brin de
           l'helice correspondante (a ete initialise dans 'ReadHelix').

===============================================================================
*/
void GetCfg(Config *cfg, Mask *mask, short *gaps_cfg)
{
  int  i, j, h2indx, dist, gapsum, min_dist, offset,

       stindx, hxindx;        /* indices dans les tab. de brins et d'helices */

  offset = mask->atom - mask->pattern->atom;


             /* --- REINITIALISATION DES PARAMETRES VARIABLES DES ATOMES --- */


  mask->atom[0].len = mask->atom[0].min_len;          /* 1er atome du masque */
  mask->atom[0].gaps = 0;
  mask->atom[0].bgn  = 0;

  for (i = 1; i < mask->natom; i++)                   /* les atomes suivants */
  {
      mask->atom[i].gaps = 0;
      mask->atom[i].len = mask->atom[i].min_len;
      mask->atom[i].bgn = mask->atom[i-1].bgn + mask->atom[i-1].len;
  }


 /* -- ACTUALISATION DES ATOMES DU MASQUE A L'AIDE DE LA CONFIG. DES GAPS -- */


  for (i = j = gapsum = 0; i < mask->natom; i++)
  {
      mask->atom[i].bgn += gapsum;
                                          /* restriction aux brins avec gaps */
      if (mask->atom[i].type == STD && mask->atom[i].max_gaps > 0)
      {
          switch (mask->atomstr[i]) {
              case '1':                       /* atome appartenant au masque */
                      mask->atom[i].gaps = gaps_cfg[j];
                      mask->atom[i].len += gaps_cfg[j];
                      gapsum += gaps_cfg[j++];
                  break;
              case '0':                                 /* atome hors masque */
                      while (mask->atomstr[i] == '0') i++;          /* passe */
                      gapsum += gaps_cfg[j++];           /* les brins exclus */
                      i--;                     /* et s'arrete sur le dernier */
                  break;
          }
      }
  }

  cfg->len = (short) (mask->min_len + gapsum);     /* longueur de la config. */


     /* --- ACTUALISATION DE LA GEOMETRIE DES HELICES ET BRINS DU MASQUE --- */


  for (i = stindx = hxindx = 0; i < mask->natom; i++)
  {
      if (mask->atomstr[i] == '1')       /* restriction aux atomes du masque */
      {
          switch (mask->atom[i].type)
          {
              case STD:                                              /* brin */
                  cfg->stbgn[stindx]  = (short) mask->atom[i].bgn;
                  cfg->stgaps[stindx] = (short) mask->atom[i].gaps;
                  cfg->stlen[stindx]  = (short) mask->atom[i].len;
                  stindx++;
                  break;

              case HLX1:                                /* 1er brin d'helice */
                  cfg->hxbgn[hxindx] = (short) mask->atom[i].bgn;

                  h2indx = mask->atom[i].h2index - offset;      /* 2eme brin */

                  dist = mask->atom[h2indx].bgn - mask->atom[i + 1].bgn;
                  min_dist = mask->atom[h2indx].min_bgn -
                             mask->atom[i + 1].min_bgn;

                  cfg->hxdist[hxindx] = (short) dist;
                  cfg->hxgaps[hxindx] = (short) (dist - min_dist);
                  hxindx++;
                  break;
              case HLX2:
                  break;
              default:
                  fprintf(stderr,
                          "GetCfg: invalid 'atom.type' for arg 2, exit..\n");
                  exit(1);
          }
      }

  }
  return;
}
/*
===============================================================================
 GetMaskCfgs(): Dresse le tableau des configurations possibles du masque pointe
                par 'mask' a partir des configurations de gaps trouvees par la
                fonction 'GetGapsCfgs' qui est appelee.
                Le masque doit avoir ete completement initialise au prealable.
                Le tableau retourne doit posseder 'mask->ncfg' elements, un
                controle est effectue.
===============================================================================
*/
void GetMaskCfgs(Mask *mask)
{
  int     i, cfgs;

  if (mask->cfg != NULL) {          /* s'assurer que mask->cfg a ete liberee */
      fprintf(stderr, "GetMaskCfgs: mask->cfg already allocated, exit..\n");
      exit(1);
  }
               /*  ou alors, destruction avant modification de 'mask->ncfg'  */
/*
  if (mask->cfg != NULL)  DelCfgTable(mask);
*/

  if (mask->nhx + mask->nst == 0) {
      mask->ncfg = 0;
      return;                                              /* --- sortie --- */
  }

  cfgs = GetGapsCfgs(mask);             /* calcul des configurations de gaps */

  if (cfgs != mask->ncfg) {                                      /* controle */
      fprintf(stderr, "GetMaskCfgs: %d config. expected, %d found! exit..\n",
              mask->ncfg, cfgs);
      exit(1);
  }

  SetCfgTable(mask);             /* allocation du tableau des configurations */

  for (i = 0; i < mask->ncfg; i++)               /* configurations du masque */
      GetCfg(mask->cfg + i, mask, mask->gapscfg[i]);

  return;
}
/*
===============================================================================
 GetMaskCfgLen(): Depuis les donnees d'une configuration pointee par 'cfg' pi-
                  lotee par un masque pointe par 'mask' retourne la longueur
                  de cette configuration.
            note: Les longueurs des configurations ont ete determinees lors de
                  leur creation.
===============================================================================
*/
int GetMaskCfgLen(Config *cfg, Mask *mask)
{
  int   *m, i, pos2, min_pos1, max_pos2;

  min_pos1 =  100000;
  max_pos2 = -100000;

  for (i = 0, m = mask->hxindex; i < mask->nhx; i++, m++)
  {
      if (cfg->hxbgn[i] < min_pos1)  min_pos1 = cfg->hxbgn[i];

      pos2 = cfg->hxbgn[i] + 2 * mask->pattern->hlx[*m].helix_len +
             cfg->hxdist[i];                             /* fin du motif + 1 */

      if (pos2 > max_pos2)  max_pos2 = pos2;
  }
  for (i = 0; i < mask->nst; i++)
  {
      if (cfg->stbgn[i] < min_pos1)  min_pos1 = cfg->stbgn[i];

      pos2 = cfg->stbgn[i] + cfg->stlen[i];              /* fin du motif + 1 */

      if (pos2 > max_pos2)  max_pos2 = pos2;
  }

  return (max_pos2 - min_pos1);
}
/*
===============================================================================
 GetMaskCfgsLens(): Applique 'GetMaskCfgLen' a l'ensemble des configurations as-
                    sociees au masque pointe par l'argument, les longueurs cal-
		    culees sont chargees dans le champ 'len' des configurations.
===============================================================================
*/
void GetMaskCfgsLens(Mask *mask)
{
  int  i;

  for (i = 0; i < mask->ncfg; i++)
      mask->cfg[i].len = (short) GetMaskCfgLen(mask->cfg + i, mask);

  return;
}
/*
===============================================================================
*/
