
/*=============================================================================
 sum.c                                          A.Lambert le 15/04/04

 Fonctions concernant la lecture et l'utilisation d'une matrice de substitution.

 cc -O2 -Wall -c sum.c -I../include ;

 ar -rs ../lib/librnaIV.a sum.o ;

=============================================================================*/

#include "rnaIV.h"

void ReadSUM(char *fname, int pcmode, double hpcw, double spcw, Trset *trset);
void SumImg(double **Sbstm, int dim, double **prof, int len, double weight);


/*=============================================================================
 ReadSUM(): L'argument 'pcmde' peut prendre les valeurs 'PC_OFF', 'PC_DEFAULT'
            ou 'PC_USER', toute autre valeur provoque la sortie.
            'pcmode' est egal a 'PC_OFF': aucune matrice de substitution n'est
            utilisee.
	    'pcmode' est egal a 'PC_DEFAULT': les donnees par defaut enregis-
	    trees dans 'libsrc/defaultSUM.c' sont utilisees.
	    'pcmode' est egal a 'PC_USER': les donnees sont lues dans le fi-
	    chier pointe par 'fname'.
	    Dans ce cas les donnees sont chargees dans 2 tableaux pointes par
	    'trset->hlxsum' et 'trset->stsum'.
  	    Les poids 'hpcw' et 'spcw' des pseudo-comptes sont enregistres.
=============================================================================*/

void ReadSUM(char *fname, int pcmode, double hpcw, double spcw, Trset *trset)
{
  int  i, j, ret;
  FILE *txt;
  extern double DefaultHlxSum[SQR_ALPHA_LEN][SQR_ALPHA_LEN];
  extern double DefaultStSum[ALPHA_LEN][ALPHA_LEN];

  if (pcmode == PC_OFF)                             /* pas de pseudo-comptes */
  {
      trset->pcmode = PC_OFF;
      trset->hpcw = trset->spcw = 0.0;
      return;
  }

  trset->hlxsum = dMat(SQR_ALPHA_LEN, SQR_ALPHA_LEN);          /* allocation */
  trset->stsum  = dMat(ALPHA_LEN, ALPHA_LEN);
  trset->hpcw = hpcw;
  trset->spcw = spcw;

  if (pcmode == PC_DEFAULT)                 /* saisie des donnees par defaut */
  {
      trset->pcmode = PC_DEFAULT;

      for (i = 0; i < SQR_ALPHA_LEN; i++)    /* matrice relative aux helices */
          for (j = 0; j < SQR_ALPHA_LEN; j++)
	      trset->hlxsum[i][j] = DefaultHlxSum[i][j];

      for (i = 0; i < ALPHA_LEN; i++)          /* matrice relative aux brins */
          for (j = 0; j < ALPHA_LEN; j++)
	      trset->stsum[i][j] = DefaultStSum[i][j];
      return;
  }

  if (pcmode == PC_USER)                 /* lecture des donnees dans 'fname' */
  {
      trset->pcmode = PC_USER;

      if ((txt = fopen(fname, "r")) == NULL)  {
          fprintf(stderr, "ReadSUM: file '%s' not found, exit..\n", fname);
          exit(1);
      }
      for (i = ret = 0; i < SQR_ALPHA_LEN; i++)  /* mat. relative aux helices */
          for (j = 0; j < SQR_ALPHA_LEN; j++)
              ret += fscanf(txt, "%lf", trset->hlxsum[i] + j);

      if (ret != SQR_ALPHA_LEN * SQR_ALPHA_LEN) {
          fprintf(stderr, "ReadSUM: 'fscanf' error #1, exit..\n");
          exit(1);
      }
      for (i = ret = 0; i < ALPHA_LEN; i++)       /* mat. relative aux brins */
          for (j = 0; j < ALPHA_LEN; j++)
              ret += fscanf(txt, "%lf", trset->stsum[i] + j);

      if (ret != ALPHA_LEN * ALPHA_LEN) {
          fprintf(stderr, "ReadSUM: 'fscanf' error #2, exit..\n");
          exit(1);
      }

      fclose(txt);
      return;
  }
  fprintf(stderr, "ReadSUM: invalid argument #2, exit..\n");
  exit(2);
}
/*=============================================================================
 SumImg(): Calcule l'image d'un profil d'helice ou de brin, suivant la dimen-
           sion 'dim' de la matrice de substitution pointee par 'Sbstm',
	   constitue de 'len' colonnes.
           Le parametre 'weight' donne le poids (entre 0 et 1) des pseudo-comp-
	   tes.
=============================================================================*/

void SumImg(double **Sbstm, int dim, double **prof, int len, double weight)
{
  int     i, j, k;
  double  *tmp;

  if (weight < 0 || weight > 1) {
      fprintf(stderr, "SumImg: arg #4 must be > 0 and < 1, exit..\n");
      exit(1);
  }

  tmp = (double *) malloc(dim * sizeof(double));

  for (j = 0; j < len; j++)             /* boucle sur les colonnes du profil */
  {
      FilldTab(tmp, dim, 0.0);
      for (i = 0; i < dim; i++)
          for (k = 0; k < dim; k++)  tmp[i] += Sbstm[i][k] * prof[k][j];

      for (i = 0; i < dim; i++)
          prof[i][j] = (1. - weight) * prof[i][j] + weight * tmp[i];
  }
  free(tmp);
  return;
}
/*===========================================================================*/
