
/*=============================================================================
 xgapslist.c                                      A.Lambert le 09/04/05

 test des fonctions qui creent et manipulent les tableaux 'gapslist' de masques
 d'une meme region.
 Les arguments 'region' et 'masques' sont entres comme pour 'erpin'.

 cc -O3 -Wall -o xgapslist xgapslist.c -I../include -L../lib -lrnaIV -lm ;
 chmod 755 xgapslist ;
 strip xgapslist ;

 usage:
 xgapslist <trset> <region> [-seed <seed>]
       -nomask|((-mask|-umask|-add) <elt1> <elt2>..)     level 1,
       [-nomask|((-mask|-umask|-add) <elt1> <elt2>..)]   level 2, default: void
       [...]                                             level..           idem

 ../bin/tview ~/devc/projets/bio/data/trsetsII/trna.db -2,2 | more ;

 HHHHHHH SS HHHH SSSSSSSSSSS HHHH S HHHHH SSSSSSS HHHHH SSSSS HHHHH SSSSSSS HHHHH HHHHHHH
 0000000 00 0000 00000000000 0000 0 00000 0000000 00000 00000 22222 1111111 22222 0000000
 2222222 33 4444 55555555555 4444 6 77777 8888888 77777 99999 00000 1111111 00000 2222222
 ******* *- **** ***--***--* **** * ***** ******* ***** ***-* ***** ******- ***** *******
 GTCCCCT TC GTCT AGA--GGCCCA GGAC A CCGCC CTTTCAC GGCGG TAA-C AGGGG TTCGAAT CCCCT AGGGGAC
 GGGCTCG TA GCTC AGC--GGG--A GAGC G CCGCC TTTGCGA GGCGG AGGCC GCGGG TTCAAAT CCCGC CGAGTCC

 xgapslist ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -umask 2 ;
 xgapslist ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -umask 2 -add 8 ;  pas de gaps!
 xgapslist ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -umask 2 -add 9 ;     des gaps!
 xgapslist ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -umask 2 -add 5 -add 8 -nomask;
 xgapslist ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -umask 7 8 9 20 11 ;
 xgapslist ~/devc/projets/bio/data/trsetsII/trna.db -4,4 -nomask ;

 xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 -nomask ;
 xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 -umask 9 ;
 xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 -umask 9 -add 65 ;
 xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 \
           -umask 9 -add 65 -add 61 ;
 xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 \
           -umask 9 -add 65 -add 61 -add 14 ;
 xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 \
           -umask 9 -add 65 -add 61 -add 14 -nomask;

 xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 -umask 9 61 ;
 xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 -umask 9 61 -add 65 ;
 xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 -umask 9 61 65 ;

 xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 \
           -umask 9 61 -add 65 -add 15 ;
 xgapslist ~/devc/projets/bio/data/trsetsII/trna.db -2,2 \
             -umask 2 -add 5 -add 8 -nomask -seed 4567555 ;

 xgapslist ~/devc/projets/bio/data/trsetsII/trna.db -4,20 -nomask ;

 xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 \
           -umask 9 61 -add 65 -add 15 ;

 xgapslist ~/devc/projets/bio/data/trsetsII/trna.db 8,8 -umask 8 ;

 xgapslist ~/devc/projets/bio/data/trsetsII/gcvt.epn -2,2 \
           -umask 3 -add 2 -add 4 -add 6 ;

=============================================================================*/

#include "rnaIV.h"

#ifdef ZZZZZZZZZZZ

#define AT_INDEX    0
#define GR_INDEX    1
#define STATUS      2
#define AT_MINGAPS  3
#define AT_MAXGAPS  4
#define GR_MINGAPS  5
#define GR_MAXGAPS  6
#define STR_INDEX   7
#define MGAPLIST_H  8

                   /* structure destinee a manipuler les groupes de brins avec
		      gaps tels qu'ils apparaissent dans le tableau 'gapslist'
		      associe a un masque.
		      A 'gapslist' on associera un tableau de ces structures */
typedef struct {
    int bgn,   /* position du 1er elt du groupe parmi les col. de 'gapslist' */
        atom,                   /* index de l'atome du 1er element du groupe */
	size,                   /* nombre de colonnes occupees par le groupe */
	min_gaps,                   /* nombre minimal de gaps dans le groupe */
	max_gaps,                   /* nombre maximal de gaps dans le groupe */
	stress_num;    /* indice 0,1,2.. d'une contrainte associee, sinon -1 */
    }
    GapsGrp;

#endif

void  PrintAllCfgs(Mask *m);
void  PrintGapsList(Mask *m);
void  printcfg(Mask *mask, short *cfg);
void  printsmat(short **m, int nl, int nc);
void  printGrps(Mask *mask);

/*===========================================================================*/

int main(int argc, char *argv[])
{
  Trset   *trset;
  Pattern *pattern;
  Mask    *mask;
  int     i, nmask, width;
  short   *cfg;

  if (argc < 3) {
      fprintf(stderr, "'%s' needs at least 2 arg, exit..\n", argv[0]);
   exit(1);
  }
  fTest(argv[1]);

  for (i = 2; i < argc - 1; i++) {
      if (strcmp(argv[i], "-seed") == 0) srand(atoi(argv[++i]));
  }

  mask = ReadMasksArgs(argc, argv, &nmask);               /* initialisations */
  trset = ReadTrset(argv[1], 'S', stdout);
  pattern = ReadPattern(argv[2], trset);
  ParseMasksArgs(mask, nmask, pattern);
  GetMasks(mask, nmask, pattern);

  width = pattern->nvarst;
  printf("nvarst=%d\n", width);

  printsmat(mask[0].gapslist, MGAPLIST_H, width + 2);
  printf("\n");
  PrintAllCfgs(mask);

  for (i = 1; i < nmask; i++)
  {
      cfg = DmpRndCfg(mask + i - 1);         /* tirage d'1 config. aleatoire */
      ImportCfgInfos(mask + i - 1, cfg);
      printcfg(mask + i - 1, cfg);
      free(cfg);

      printf("level %d -> %d\n", i - 1, i);

      PrintGapsList(mask + i);

      ExportMaskInfos(mask + i - 1, mask + i);
      PrintGapsList(mask + i);
      PrintAllCfgs(mask + i);
  }
  DelTrset(trset);
  DelMasks(mask, nmask);
  DelPattern(pattern);
  return 0;
}
/*=============================================================================
 PrintAllCfgs():
 Ex:  PrintAllCfgs(m + i)
=============================================================================*/
void  PrintAllCfgs(Mask *m)
{
  int    n, h;

  printf("################### All Cfgs ####################\n");

  n = GetGapsCfgs(m);
  printf("total cgf: %d\n", m->ncfg);
  h = ( n > 20 ? 20 : n);                    /* affichage limite a 20 lignes */

  printsmat(m->gapscfg, h, m->ngrps);
  if (h < n) printf(" ...\n");

  printf("#################################################\n");
}
/*=============================================================================
 PrintGapsList():
=============================================================================*/
void PrintGapsList(Mask *m)
{
  int width, height;

  width = m->pattern->nvarst;
  height = MGAPLIST_H + m->nstress;

  printsmat(m->gapslist, height, width + 2);
  printf("\n");
}
/*=============================================================================
 printcfg(): Affiche les nb. de gaps d'une configuration de masque.
=============================================================================*/
void printcfg(Mask *mask, short *cfg)
{
  int j;

  printf("RndCfg:");
  for (j = 0; j < mask->ngrps; j++) printf("  %2d", cfg[j]);
  printf("\n");
}
/*=============================================================================
 printsmat():
=============================================================================*/
void printsmat(short **m, int nl, int nc)
{
  int i, j;

  if (m == NULL) {
      fprintf(stderr, "printsmat: arg. pointer not allocated...\n");
      return;
  }
  printf("\n");
  for (i = 0; i < nl; i++) {
      for (j = 0; j < nc; j++)
          printf("%2d  ", m[i][j]);
      printf("\n");
  }
}
/*=============================================================================
 printGrps():
=============================================================================*/
void printGrps(Mask *mask)
{
  int i;

  for (i = 0; i < mask->ngrps; i++)
  {
      printf("grp[%d]:\n", i);
      printf("bgn=%d\n", mask->grp[i].bgn);
      printf("atom=%d\n", mask->grp[i].atom);
      printf("size=%d\n", mask->grp[i].size);
      printf("min_gaps=%d\n", mask->grp[i].min_gaps);
      printf("max_gaps=%d\n\n", mask->grp[i].max_gaps);
      printf("stress_num=%d\n\n", mask->grp[i].stress_num);
  }
}
/*===========================================================================*/

/*
erpin$ xgapslist ~/devc/projets/bio/data/trsetsIII/rnasep2.epn 67,31 \
                 -umask 9 61 -add 65 -add 15 ;

Training set:   "/home/lambert/devc/projets/bio/data/trsetsIII/rnasep2.epn":
                310 sequences of length 1570
nvarst=19

 0   2   6   8  10  11  12  13  14  15  16  17  18  19  20  21  22  24  28  19   0
-1  -1  -1   0   1   1   1   1   1   1   1   1   1   1   2  -1  -1  -1  -1   3   0
 0   0   0   1  10  10  10  10  10  10  10  10  10  10   1   0   0   0   0   0   0
 0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
11   6   1   1   1   1  16   2  18   2   1   1  36   1  38   1   1   4   4   0   0
 0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
 0   0   0   0  79  79  79  79  79  79  79  79  79  79   0   0   0   0   0   0   0
-1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1   0   0

################### All Cfgs ####################
total cgf: 6240

 0   0   0
 0   0   1
 0   0   2
 0   0   3
 0   0   4
 0   0   5
 0   0   6
 0   0   7
 0   0   8
 0   0   9
 0   0  10
 0   0  11
 0   0  12
 0   0  13
 0   0  14
 0   0  15
 0   0  16
 0   0  17
 0   0  18
 0   0  19
 ...
#################################################
RndCfg:   1  31  30
level 0 -> 1

 0   2   6   8  10  11  12  13  14  15  16  17  18  19  20  21  22  24  28  19   0
-1  -1  -1   0   1   1   1   1   2   3   3   3   3   3   4  -1  -1  -1  -1   5   0
 0   0   0   1   4   4   4   4   1   5   5   5   5   5   1   0   0   0   0   1   0
 0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
11   6   1   1   1   1  16   2  18   2   1   1  36   1  38   1   1   4   4   0   0
 0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
 0   0   0   0  20  20  20  20   0  41  41  41  41  41   0   0   0   0   0   0   0
-1  -1  -1  -1   0  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1   1   0
 0   0   0   0   1   0   0   0   1   1   0   0   0   0   0   0   0   0   0   1   0


 0   2   6   8  10  11  12  13  14  15  16  17  18  19  20  21  22  24  28  19   0
-1  -1  -1   0   1   1   1   1   2   3   3   3   3   3   4  -1  -1  -1  -1   5   0
 0   0   0   1   4   4   4   4   1   5   5   5   5   5   1   0   0   0   0   1   0
 0   0   0   1   0   0   0   0   0   0   0   0   0   0  30   0   0   0   0   0   0
11   6   1   1   1   1  16   2  18   2   1   1  36   1  30   1   1   4   4   0   0
 0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
 0   0   0   0  20  20  20  20   0  41  41  41  41  41   0   0   0   0   0   0   0
-1  -1  -1  -1   0  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1   1   0
 0   0   0   0   1   0   0   0   1   1   0   0   0   0   0   0   0   0   0   1  31

################### All Cfgs ####################
total cgf: 371

 1  20  11   0  30
 1  19  12   0  30
 1  18  13   0  30
 1  17  14   0  30
 1  16  15   0  30
 1  15  16   0  30
 1  14  17   0  30
 1  13  18   0  30
 1  20  10   1  30
 1  19  11   1  30
 1  18  12   1  30
 1  17  13   1  30
 1  16  14   1  30
 1  15  15   1  30
 1  14  16   1  30
 1  13  17   1  30
 1  12  18   1  30
 1  20   9   2  30
 1  19  10   2  30
 1  18  11   2  30
 ...
#################################################
RndCfg:   1  13  13   5  30
level 1 -> 2

 0   2   6   8  10  11  12  13  14  15  16  17  18  19  20  21  22  24  28  19   0
-1  -1  -1   0   1   1   2   3   4   5   5   5   5   5   6  -1  -1  -1  -1   7   0
 0   0   0   1   2   2   1   1   1   5   5   5   5   5   1   0   0   0   0   1   0
 0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
11   6   1   1   1   1  16   2  18   2   1   1  36   1  38   1   1   4   4   0   0
 0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0   0
 0   0   0   0   2   2   0   2   0  41  41  41  41  41   0   0   0   0   0   0   0
-1  -1  -1  -1   0  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1   1   0
 0   0   0   0   1   0   1   1   0   0   0   0   0   0   0   0   0   0   0   1   0


 0   2   6   8  10  11  12  13  14  15  16  17  18  19  20  21  22  24  28  19   0
-1  -1  -1   0   1   1   2   3   4   5   5   5   5   5   6  -1  -1  -1  -1   7   0
 0   0   0   1   2   2   1   1   1   5   5   5   5   5   1   0   0   0   0   1   0
 0   0   0   1   0   0   0   0  13   0   0   0   0   0  30   0   0   0   0   0   0
11   6   1   1   1   1  16   2  13   2   1   1  36   1  30   1   1   4   4   0   0
 0   0   0   0   0   0   0   0   0   5   5   5   5   5   0   0   0   0   0   0   0
 0   0   0   0   2   2   0   2   0   5   5   5   5   5   0   0   0   0   0   0   0
-1  -1  -1  -1   0  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1  -1   1   0
 0   0   0   0   1   0   1   1   0   0   0   0   0   0   0   0   0   0   0   1  13

################### All Cfgs ####################
total cgf: 9

 1   2  11   0  13   5  30
 1   1  12   0  13   5  30
 1   0  13   0  13   5  30
 1   2  10   1  13   5  30
 1   1  11   1  13   5  30
 1   0  12   1  13   5  30
 1   2   9   2  13   5  30
 1   1  10   2  13   5  30
 1   0  11   2  13   5  30
#################################################

*/
