
/*=============================================================================
 mcfgs.c                            A.Lambert le 25/01/02 revu le 10/03/05

 Teste les fonctions concernant la localisation des atomes d'un masque, les
 codant par 1 dans une chaine ou sont enumeres les atomes de l'enveloppe du
 masque (le complementaire est code par 0).
 Teste aussi les fonctions concernant la construction des configurations d'un
 masque dans une region (chaque masque est considere separement des autres:
 les contraintes qui les lient dans une operation de recherche ne sont pas 
 prises en compte).

 cc -O3 -Wall -o mcfgs mcfgs.c -I../include -L../lib -lrnaIV -lm;
 chmod 755 mcfgs ;
 strip mcfgs ;

 usage:
 mcfgs <trset> <region>
       -nomask|((-mask|-umask|-add) <elt1> <elt2>..)     level 1,
       [-nomask|((-mask|-umask|-add) <elt1> <elt2>..)]   level 2, default: void
       [...]                                             level..           idem
       [-V]                    mode verbeux, affichage des configs, defaut: non


 mcfgs ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -mask 9 20 11 -V ;
 mcfgs ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -umask 9 -add 2 -add 20
 mcfgs ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -mask 20 -umask 2 -nomask ;
 mcfgs ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -umask 2 ;
 mcfgs ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -umask 2 7 8 9 20 11 ;
 mcfgs ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -mask 2 9 20 11 ;
 mcfgs ~/devc/projets/bio/data/trsetsII/trna.db 1,12 -mask 9 20 11 ;
 mcfgs ~/devc/projets/bio/data/trsetsII/trna.db 1,12 -mask 2 9 20 11 ;
 mcfgs ~/devc/projets/bio/data/trsetsII/trna98.db 1,12 -mask 2 9 20 11 -nomask ;
 mcfgs ~/devc/projets/bio/data/trsetsII/trna.db -2,2 -umask 7 20 ;
                                             exemple avec 1 seul brin sans gap !
 mcfgs ~/devc/projets/bio/data/trsetsII/trna.db 8,8 -umask 8 -V ;

=============================================================================*/

#include "rnaIV.h"

void printfmask(Mask *mask, int n);
void printcfgs(Mask *M);

/*===========================================================================*/

int main(int argc, char *argv[])
{
  Trset   *trset;
  Pattern *pattern;
  Mask    *mask;
  int     i, verbose = 0, nmask;

  if (argc < 3) {
      fprintf(stderr, "'%s' needs at least 2 arg, exit..\n", argv[0]);
   exit(1);
  }
  for (i = 3; i < argc; i++)
      if (strcmp(argv[i], "-V") == 0) verbose = 1;

  fTest(argv[1]);

  mask = ReadMasksArgs(argc, argv, &nmask);
  trset = ReadTrset(argv[1], 'S', stdout);
  pattern = ReadPattern(argv[2], trset);
  ParseMasksArgs(mask, nmask, pattern);

  for (i = 0; i < nmask; i++)
  {
      GetMask(mask + i, pattern);
      printfmask(mask, i);
      if (verbose) {
          GetMaskCfgs(mask + i);
          printcfgs(mask + i);
          DelCfgTable(mask + i);
      }
      FreeMask(mask + i);
  }

  DelTrset(trset);
  DelMasks(mask, nmask);
  DelPattern(pattern);
  exit(0);
}
/*=============================================================================
 printfmask(): affiche les resultats de la lecture des atomes du masque No 'n'.
=============================================================================*/

void printfmask(Mask *mask, int n)
{
  int   i, j, width;
  Mask  *m = mask + n;
  
  width = m->pattern->nvarst;

  printf("\nstep #%d: %d atoms\n", n+1, m->natom);
  printf("mask: %s\n", m->atomstr);
  printf("gaps: ");
  for (i = 0; i < m->natom; i++) printf("%d ", m->atom[i].max_gaps);
  printf("\n");

  printf("mask.ngrps: %d\n", m->ngrps);
  printf("mask.gapslist:\n");

  if (m->gapslist != NULL) {
      for (i = 0; i < MGAPLIST_H; i++) {
          for (j = 0; j < width+1; j++)
              printf("%2d  ", m->gapslist[i][j]);
          if (i != MGAPLIST_H - 1) printf("\n");
      }
  }
  printf(" -> %d configs.\n", m->ncfg);
/*
  PrintMaskCfgsVol(m, stdout);
*/
  return;
}
/*=============================================================================
 printcfgs(): Imprime les donnees du tableau des configurations d'un masque.
=============================================================================*/

void printcfgs(Mask *M)
{
  int i, j;

 printf("\n--------------------------------------------------\n");
 printf("Format:\n");
 printf("helices: bgn gaps dist * ...\n");
 printf("strands: bgn gaps len  * ...");

  for (i = 0; i < M->ncfg; i++)
  {
      printf("\n--------------------------------------------------\n");
      printf("%-4d len: %d\n", i+1, M->cfg[i].len);
      printf("helices: ");
      for (j = 0; j < M->nhx; j++) {
          printf("%d %d %d * ",
                 M->cfg[i].hxbgn[j], M->cfg[i].hxgaps[j], M->cfg[i].hxdist[j]);
      }
      printf("\nstrands: ");
      for (j = 0; j < M->nst; j++) {
          printf("%d %d %d * ",
                 M->cfg[i].stbgn[j], M->cfg[i].stgaps[j], M->cfg[i].stlen[j]);
      }
  }
  printf("\n\n");
  return;
}
/*===========================================================================*/
